@@ -1,14 +1,17 @@
+package Archive::Zip::MockFileHandle;
+
 # Output file handle that calls a custom write routine
 # Ned Konz, March 2000
 # This is provided to help with writing zip files
 # when you have to process them a chunk at a time.
-#
-# See the examples.
-#
-# $Revision: 1.2 $
 
 use strict;
-package Archive::Zip::MockFileHandle;
+
+use vars qw{$VERSION};
+BEGIN {
+	$VERSION = '1.17_01';
+	$VERSION = eval $VERSION;
+}
 
 sub new
 {
@@ -28,19 +31,11 @@ sub eof
 }
 
 # Copy given buffer to me
-sub write
+sub print
 {
 	my $self = shift;
-	my $buf = \($_[0]); shift;
-	my $len = shift;
-	my $offset = shift || 0;
-
-	$$buf = '' if not defined($$buf);
-	my $bufLen = length($$buf);
-	my $bytesWritten = ($offset + $len > $bufLen)
-		? $bufLen - $offset
-		: $len;
-	$bytesWritten = $self->writeHook(substr($$buf, $offset, $bytesWritten));
+	my $bytes = join('', @_);
+	my $bytesWritten = $self->writeHook($bytes);
 	if ($self->{'position'} + $bytesWritten > $self->{'size'})
 	{
 		$self->{'size'} = $self->{'position'} + $bytesWritten
@@ -70,5 +65,7 @@ sub read { 0 }
 
 sub tell { return shift->{'position'} }
 
+sub opened { 1 }
+
 # vim: ts=4 sw=4
 1;
